/*
 * Copyright (c) 2004 Steve Northover and Mike Wilson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * 
 */
package part1.ch11;

import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;

public class Shell1 {

public static void main(String[] args) {
    final Display display = new Display();
    final Shell shell = new Shell(display);
    shell.setLayout(new FillLayout());
    Listener tipListener = new Listener() {
        Shell tip = null;
        Label label = null;
        void createToolTip() {
            tip = new Shell(shell, SWT.ON_TOP);
            tip.setLayout(new FillLayout());
            label = new Label(tip, SWT.NONE);
            Listener listener = new Listener() {
                public void handleEvent(Event event) {
                    tip.dispose();
                }
            };
            label.addListener(SWT.MouseExit, listener);
            Color foreground =
                display.getSystemColor(
                    SWT.COLOR_INFO_FOREGROUND);
            label.setForeground(foreground);
            Color background =
                display.getSystemColor(
                    SWT.COLOR_INFO_BACKGROUND);
            label.setBackground(background);
            label.setText("ToolTip");
            tip.pack();
        }
        public void handleEvent(Event e) {
            switch (e.type) {
                case SWT.KeyDown:
                case SWT.MouseMove:
                    if (tip != null) tip.dispose();
                    tip = null;
                    break;
                case SWT.MouseHover: {
                    if (tip != null) break;
                    createToolTip();
                    Rectangle rect = tip.getBounds();
                    rect.x = e.x;
                    rect.y = e.y + 22;
                    tip.setBounds(
                        display.map(shell, null, rect));
                    tip.setVisible(true);
                }
            }
        }
    };
    shell.addListener(SWT.KeyDown, tipListener);
    shell.addListener(SWT.MouseHover, tipListener);
    shell.addListener(SWT.MouseMove, tipListener);
    shell.pack();
    shell.open();
    while (!shell.isDisposed()) {
        if (!display.readAndDispatch()) display.sleep();
    }
    display.dispose();
}
}
